.tbbtn{
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0;
    border: 1px solid rgba(125,125,125,0.5);
}
.tbbtn.actv{
    box-shadow: inset 0 0 5px 10px rgba(125,125,125,0.5);
}
.tbcont{
    flex-grow:1;
}
._tb_{
    border:1px solid rgba(125,125,125,0.5);
    height:100%;
    width:100%;
    overflow-y:auto;
    overflow-x:auto;
}